
#include "pm.h"
#include "small_chars.h"
#include "ram_code.h"
#include "patch_roms.h"

#include <string.h>
#include <stdint.h>


#define NINTENDO_OFFSET 0x21A4
#define GAME_NAME_OFFSET 0x21AC



struct GAME {
    uint8_t pos;
    char name[20];
} game[32];
int games;


static void switch_2m(void) {
    BANK_SWITCH = 0x00;
}

static void switch_512k_0(void) {
    BANK_SWITCH = 0x80;
}

static uint8_t get_key(void) {
    uint8_t key1 = KEY_PAD, key2 = KEY_PAD;
    while (key1 != key2) {
        key1 = key2;
        key2 = KEY_PAD;
    }
    return key2;
}

volatile uint8_t flag;
_interrupt(2) void prc_frame_copy_irq(void)
{
    flag = 1;
    IRQ_ACT1 = IRQ1_PRC_COMPLETE;
}

void display_list(uint8_t start) {
    uint8_t i;
    clear_screen();
    for (i = 0; i < 8; i++) {
        if (start + i >= games) break;
        printn(6, i, game[start + i].name, 4);
        print(32, i, game[start + i].name + 4);
    }
}

#define GAME_NAMES 10
const char _rom *game_names[GAME_NAMES][2] = {
    {"MZ2J", "Puzzle Coll2"},
    {"MPZJ", "PuzzleCol(J)"},
    {"MTAJ", "Togepi Adven"},
    {"MBRJ", "Pichu Bros  "},
    {"MPBJ", "Pinball (J) "},
    {"MRCJ", "Pokemon Race"},
    {"MPTJ", "Pkm Party(J)"},
    {"MSDJ", "Breeder (J) "},
    {"MACJ", "ZanyCards(J)"},
    {"MSTJ", "Tetris (J)  "},
};

void fix_display_name(char *name) {
    uint8_t i;
    for (i = 0; i < GAME_NAMES; ++i) {
        if (strncmp(name, game_names[i][0], 4) == 0) {
            strcpy(name + 4, game_names[i][1]);
            return;
        }
    }
}

#define KEY_SPEED 7
int main(void) {
    uint8_t i, n, start;
    uint32_t i2;
    uint8_t keys;
    uint8_t cnt = 0;

    // TODO: disable all interrupts temporary
    // disable key interrupts
    IRQ_ENA3 = 0;

    switch_2m();

    games = 0;
    for (i2 = 2; i2 < 32; i2++) {
        if (strncmp(((i2 << 16) + NINTENDO_OFFSET), "NINTENDO", 8) == 0) {
            game[games].pos = (uint8_t)i2;
            memcpy(game[games].name, ((i2 << 16) + GAME_NAME_OFFSET), 16);
            game[games].name[16] = 0;
            fix_display_name(game[games].name);
            patch_rom(i2 << 16);
            ++games;
        }
    }
/*
    while (get_key() & KEY_B)
        ;
*/
    // key interrupts priority
    PRI_KEY(0x03);
    // enable interrupts for keys (only power)
    IRQ_ENA3 = IRQ3_KEYPOWER;

    // PRC interrupt priority
    PRI_PRC(0x01);
    // Enable PRC IRQ
    IRQ_ENA1 = IRQ1_PRC_COMPLETE;

    n = 0;
    start = 0;
    display_list(start);
    for(;;) {
        flag = 0;
        while(!flag)
            ;
        if (cnt > 0) --cnt;
        keys = get_key();
        print(0, n, " ");
        print(91, n, " ");
        if (!(keys & KEY_A)) {
            i = game[start + n].pos;
            if ((i & 0x07) == 0) {
                switch_memory_bank(0x80 | (i >> 3));
            } else {
                switch_memory_bank(i);
            }
        }
        if (!(keys & KEY_UP) && cnt == 0 && start + n > 0) {
            if (n > 0)
                --n;
            else {
                --start;
                display_list(start);
            }
            cnt = KEY_SPEED;
        }
        if (!(keys & KEY_DOWN) && cnt == 0 && start + n < games - 1) {
            if (n < 7)
                ++n;
            else {
                ++start;
                display_list(start);
            }
            cnt = KEY_SPEED;
        }
        print(0, n, ">");
        print(93, n, "<");
    }
}

// sleep mode fixes:
// tetris fixes work but not the rest!
const uint8_t _rom  togepi_sleepmode_fix[7] _at(0x2A3F) = { 0xff, 0xff, 0xff, 0xff, 0xff, /*0xce, 0xc5, 0x00,*/ 0xfc, 0x02 };
//const uint8_t _rom  togepi_sleepmode_fix[2] _at(0x2A41) = { 0xfc, 0x02 };
const uint8_t _rom    race_sleepmode_fix[2] _at(0x2674) = { 0xfc, 0x02 };
const uint8_t _rom  tetris_sleepmode_fix[2] _at(0x4B19) = { 0xfc, 0x48 };
const uint8_t _rom tetrisj_sleepmode_fix[2] _at(0x4AE4) = { 0xfc, 0x02 };
const uint8_t _rom breeder_sleepmode_fix[2] _at(0x2A89) = { 0xfc, 0x02 };
const uint8_t _rom   pichu_sleepmode_fix[2] _at(0x4199) = { 0xfc, 0x02 };

//const uint8_t _rom     end_sleepmode_fix[6] _at(0x7f000) = { 0xfc, 0x00 };
