#include "patch_roms.h"
#include "ram_code.h"

#include "small_chars.h"

#include <string.h>

#define GAME_NAME_OFFSET 0x21AC
#define SECTOR_SIZE 4096

#define FF_64   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,\
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,\
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,\
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
const uint8_t _rom scratch_buffer[SECTOR_SIZE] _at(0xF000) = {
        FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64
        FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64
        FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64
        FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64 FF_64
};

struct PatchData {
    char *name;
    uint32_t offset;
    uint8_t old_val;
    uint8_t new_val;
};

#define PATCH_DATA_NUM 8

// tetris fixes work so we don't need to patch it
const struct PatchData _rom patch_data[PATCH_DATA_NUM] = {
    { "MTAJ", 0x2A40, 0x42, 0x48 }, // togepi
    { "MRCJ", 0x2673, 0x42, 0x48 }, // race
    { "MSDJ", 0x2A88, 0x42, 0x48 }, // breeder
    { "MBRJ", 0x419A, 0x42, 0x48 }, // pichu
    { "MTAE", 0x2A40, 0x42, 0x48 }, // togepi
    { "MRCE", 0x2673, 0x42, 0x48 }, // race
    { "MSDE", 0x2A88, 0x42, 0x48 }, // breeder
    { "MBRE", 0x419A, 0x42, 0x48 }, // pichu
    //{ "MSTP", 0x4B18, 0x42, 0x48 }, // tetris e
    //{ "MSTJ", 0x4AE3, 0x42, 0x48 }, // tetris j
};

// TODO: record how far we are with the patch
static void do_patch(uint32_t address, const struct PatchData _far *patch) {
    uint32_t patch_sector = address + (patch->offset & 0xF000);
    uint16_t offset = patch->offset & 0xFFF;
    uint16_t i;
    // TODO: better prints
    print(10, 1, "Patching:");
    printn(48, 1, patch->name, 4);
    print(10, 2, "Reading:");
    // TODO: check if it's clear allready
    erase_sector(scratch_buffer);
    for (i = 0; i < offset; ++i) {
        if((i&0xf) == 0) print_hex(48, 2, i);
        write_byte(scratch_buffer + i, *(uint8_t _far *)(patch_sector + i));
    }
    write_byte(scratch_buffer + i, patch->new_val);
    for (++i; i < SECTOR_SIZE; ++i) {
        if((i&0xf) == 0) print_hex(48, 2, i);
        write_byte(scratch_buffer + i, *(uint8_t _far *)(patch_sector + i));
    }
    print(10, 3, "Writting:");
    erase_sector(patch_sector);
    for (i = 0; i < SECTOR_SIZE-1; ++i) {
        if((i&0xf) == 0) print_hex(48, 3, i);
        write_byte(patch_sector + i, scratch_buffer[i]);
    }
    // needed because of the epson compiler bug with the end of the loop!
    // optimisations changes the semantics and end condition is done wrong
    write_byte(patch_sector + i, scratch_buffer[i]);
    print(10, 4, "Done!");
    clear_screen();
}

void patch_rom(uint32_t address) {
    uint8_t i;

    print(10, 0, "Patch search...");
    for (i = 0; i < PATCH_DATA_NUM; ++i) {
        if (strncmp(address + GAME_NAME_OFFSET, patch_data[i].name, 4) == 0) {
            if (*(uint8_t _far *)(address + patch_data[i].offset) == patch_data[i].old_val)
                do_patch(address, &patch_data[i]);
            break;
        }
    }
}
