#include "ram_code.h"
#include "small_chars.h"

#include <string.h>

#define FLASH_ADDR1 (*((volatile uint8_t _far *)0x1FFAAA))
#define FLASH_ADDR2 (*((volatile uint8_t _far *)0x1FF555))

uint8_t params_in_ram[32];
uint8_t function_in_ram[1024];


static uint8_t reverse(uint8_t b) {
    b = (b & 0xF0) >> 4 | (b & 0x0F) << 4;
    b = (b & 0xCC) >> 2 | (b & 0x33) << 2;
    b = (b & 0xAA) >> 1 | (b & 0x55) << 1;
    return b;
}


static void run_in_ram(void (*f_begin)(void)) {
    static uint8_t *previous = 0;
    void (*f)(void);
    uint8_t *p1;

    p1 = (void *)f_begin;
    f = (void *)function_in_ram;
    if (p1 != previous) {
        memcpy(function_in_ram, p1, sizeof(function_in_ram));
        previous = p1;
    }

    (*f)();
}

static void switch_memory_bank_ram(void) {
    BANK_SWITCH = params_in_ram[0];
    _int(0x02);
}

void switch_memory_bank(uint8_t bank) {
    params_in_ram[0] = bank;
    run_in_ram(switch_memory_bank_ram);
}

static void erase_sector_ram(void) {
    int i;
    uint32_t addr_long;
    FLASH_ADDR1 = 0x55;
    FLASH_ADDR2 = 0xAA;
    FLASH_ADDR1 = 0x01;
    FLASH_ADDR1 = 0x55;
    FLASH_ADDR2 = 0xAA;
    addr_long = *((uint32_t *)(&params_in_ram[1]));
    *((volatile uint8_t _far *)addr_long) = 0x0a;

    for (i = 0; i<1000; ++i) {
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
        _nop();
    }
}

void erase_sector(uint32_t address) {
    //print(0,3, "Erased:");
    //print_hex_long(30,3, address);
    *((uint8_t **)(&params_in_ram[1])) = (uint8_t *)address;
    run_in_ram(erase_sector_ram);
}

static void write_byte_ram(void) {
    uint32_t addr_long;
    FLASH_ADDR1 = 0x55;
    FLASH_ADDR2 = 0xAA;
    FLASH_ADDR1 = 0x05;
    addr_long = *((uint32_t *)(&params_in_ram[1]));
    *((volatile uint8_t _far *)addr_long) = params_in_ram[10];
    _nop();
    _nop();
    _nop();
    _nop();
    _nop();
    _nop();
    _nop();
    _nop();
    _nop();
    _nop();
    _nop();
    _nop();
}

void write_byte(uint32_t address, uint8_t value) {
    *((uint8_t **)(&params_in_ram[1])) = (uint8_t *)address;
    params_in_ram[10] = value;
    run_in_ram(write_byte_ram);
}
