
#include "small_chars.h"
#include "small_chars_font.h"

#include <string.h>

int print_ch(int x, int y, unsigned char ch)
{
    int i;
    const unsigned char *p2;
    unsigned char *p = (unsigned char *)(0x1000 + x + (int)y * 96);
    short len;

    if(ch < 0x20 || ch >= 0x80)
        ch = 0;
    else
        ch -= 0x20;

    p2 = small_chars_font[ch];
    len = *p2++;
    for(i = 0; i<len; i++)
        *p++ = *p2++;
    for(; i < 6; i++)
        *p++ = 0;
    return x + len;
}

int print_digit(int x, int y, int digit)
{
    if(0<=digit && digit <=9)
        digit += '0';
    else if(0xA<=digit && digit <=0xF)
        digit += 'A' - 0xA;
    else
        digit = '?';

    return print_ch(x, y, digit);
}

int print_dec(char x, char y, int num)
{
    int base;
    if(num < 0)
    {
        print_ch(x, y, '-');
        x += 1;
        num = -num;
    }
    for(base = 10000; base > 1; base /= 10)
        if(num >= base)
            break;
    for(; base >= 1; base /= 10)
    {
        int n2 = num / base;
        x = print_digit(x, y, n2);
        num %= base;
    }
    return x;
}

int print_hex(char x, char y, unsigned num)
{
    char i;
    for(i=12; i>=0; i-=4, x+=5)
        print_digit(x, y, (num >> i) & 0x0f);
    return x;
}

int print_byte_hex(char x, char y, unsigned num)
{
    char i;
    for(i=4; i>=0; i-=4, x+=5)
        print_digit(x, y, (num >> i) & 0x0f);
    return x;
}

int print_hex_long(char x, char y, unsigned long num)
{
    char i;
    for(i=28; i>=0; i-=4, x+=5)
        print_digit(x, y, (num >> i) & 0x0f);
    return x;
}

int print(int x, int y, const char *str)
{
    for(; *str; ++str)
        x = print_ch(x, y, *str);
    return x;
}

int printn(int x, int y, const char *str, int size)
{
    for(; size; ++str, --size)
        x = print_ch(x, y, *str);
    return x;
}

void clear_screen(void) {
    memset(0x1000, 0, 96 * 8);
}
